use argh::FromArgs;

use crate::action::ActionExec;
use crate::api::{TorrentTarget, qbittorrent::ApiClient};
use crate::config::Config;
use crate::error::Error;

#[derive(FromArgs, Debug)]
#[argh(subcommand, name = "replace-tracker")]
/// replace trackers on a torrent which match a specific regex ; the new trackers are only
/// added if a match in older trackers was found
pub struct ReplaceTrackerAction {
    #[argh(positional)]
    /// the infohash for which to replace a tracker ; use "all" to replace for all torrents
    torrent: TorrentTarget,
    #[argh(positional)]
    /// a REGEX matching the tracker(s) to be replaced
    old_tracker: regex::Regex,
    #[argh(positional, greedy)]
    /// the list of new trackers to set
    new_trackers: Vec<String>,
}

impl ActionExec for ReplaceTrackerAction {
    fn exec(&self, config: &Config) -> Result<(), Error> {
        let api = ApiClient::from_config(&config)?;
        let mut errors: Vec<Error> = Vec::new();

        // Workaround for weird qBittorrent API
        let list = api.list()?;

        for entry in self.torrent.to_vec(&api)? {
            match api.get_with_cached_list(&entry, &list) {
                Ok(found_torrent) => {
                    let found_torrent = found_torrent.unwrap_or_else(|| panic!("Torrent in 'list' could not be 'get': {}", &entry));
                    match api.get_trackers(&found_torrent.hash) {
                        Ok(found_trackers) => {
                            let mut found_matching_tracker = false;

                            for found_tracker in found_trackers {
                                let found_tracker = found_tracker.url;
                                if self.old_tracker.is_match(&found_tracker) {
                                    found_matching_tracker = true;

                                    match api.remove_tracker(&found_torrent.hash, &found_tracker) {
                                        Ok(_) => {},
                                        Err(e) => {
                                            println!("Failed to remove tracker {} for {}", &found_tracker, &found_torrent.hash);
                                            errors.push(e);
                                        },
                                    }
                                }
                            }

                            // add new trackers if there was a matching tracker found
                            if found_matching_tracker {
                                for new_tracker in &self.new_trackers {
                                    match api.add_tracker(&found_torrent.hash, new_tracker) {
                                        Ok(()) => {},
                                        Err(e) => {
                                            println!("Failed to add tracker {} for {}", &new_tracker, &found_torrent.hash);
                                            errors.push(e);
                                        }
                                    }
                                }
                            }
                        }, Err(e) => {
                            println!("Failed to find trackers for torrent {}", &found_torrent.hash);
                            errors.push(e);
                        }
                    }
                }, Err(e) => {
                    println!("Failed to find info about torrent {}", entry);
                    errors.push(e);
                }
            }
        }

        if errors.is_empty() {
            Ok(())
        } else {
            Err(Error::MultipleErrors { sources: errors })
        }
    }
}
