use snafu::prelude::*;

use crate::Config;
use crate::api::{Torrent, TorrentList, IntoTorrent, TorrentTracker};
use crate::api::qbittorrent::{QBittorrentListTorrent, RawAsyncApiClient};
use crate::error::{Error, FailedDeserializeSnafu as DeserializeError};

#[derive(Debug)]
pub struct AsyncApiClient {
    pub raw_api: RawAsyncApiClient,
}

impl AsyncApiClient {
    #[allow(dead_code)]
    pub async fn from_config(config: &Config) -> Result<AsyncApiClient, Error> {
        Self::new(&config.format_host(), &config.qbittorrent.login, &config.qbittorrent.password).await
    }

    #[allow(dead_code)]
    pub async fn new(host: &str, login: &str, password: &str) -> Result<AsyncApiClient, Error> {
        let api = RawAsyncApiClient::login(host, login, password).await?;

        Ok(AsyncApiClient {
            raw_api: api,
        })
    }

    #[allow(dead_code)]
    pub async fn add(&self, magnet: &str, paused: bool) -> Result<(), Error> {
        let res = self.raw_api.add(magnet, paused).await?;
        if res == "Ok." {
            Ok(())
        } else {
            Err(Error::message(res.clone()))
        }
    }

    #[allow(dead_code)]
    pub async fn get(&self, hash: &str) -> Result<Option<Torrent>, Error> {
        let res = self.raw_api.get(hash).await?;
        if res == "" {
            Ok(None)
        } else {
            // TODO: NOT OPTIMIZED AT ALL. API DOES NOT RETURN NAME/HASH OF TORRENT SO WE HAVE TO QUERY LIST...
            let list = self.list().await?;
            let filtered_list = list.into_iter().filter(|t| t.hash == hash).collect::<Vec<Torrent>>();
            let torrent = filtered_list.first()
                .unwrap_or_else(|| panic!("Torrent was 'get' but could not be found in 'list': {}", &hash));
            Ok(Some(torrent.clone()))
        }
    }

    #[allow(dead_code)]
    /// This method only exists because qBittorrent API 'properties' endpoint does not return torrent name/hash
    /// so if you need to do a lot of 'get', please call 'list' once and use this method instead.
    pub async fn get_with_cached_list(&self, hash: &str, list: &TorrentList) -> Result<Option<Torrent>, Error> {
        let res = self.raw_api.get(hash).await?;
        if res == "" {
            Ok(None)
        } else {
            let filtered_list = list.clone().into_iter().filter(|t| t.hash == hash).collect::<Vec<Torrent>>();
            let torrent = filtered_list.first()
                .unwrap_or_else(|| panic!("Torrent was 'get' but could not be found in 'list': {}", &hash));
            Ok(Some(torrent.clone()))
        }
    }

    #[allow(dead_code)]
    pub async fn list(&self) -> Result<TorrentList, Error> {
        let res = self.raw_api.list().await?;
        let concrete: Vec<QBittorrentListTorrent> = serde_json::from_str(&res).context(DeserializeError)?;
        Ok(concrete.iter().map(|t| t.into_torrent()).collect())
    }

    #[allow(dead_code)]
    pub async fn add_tracker(&self, hash: &str, url: &str) -> Result<(), Error> {
        let _ = self.raw_api.add_tracker(hash, url).await?;
        Ok(())
    }

    #[allow(dead_code)]
    pub async fn get_trackers(&self, hash: &str) -> Result<Vec<TorrentTracker>, Error> {
        let res = self.raw_api.get_trackers(hash).await?;
        let concrete: Vec<TorrentTracker> = serde_json::from_str(&res).context(DeserializeError)?;
        let concrete_filter_dht = concrete.into_iter().filter(|t| t.is_tracker()).collect();
        Ok(concrete_filter_dht)
    }

    #[allow(dead_code)]
    pub async fn remove_tracker(&self, hash: &str, url: &str) -> Result<(), Error> {
        let _ = self.raw_api.remove_tracker(hash, url).await?;
        Ok(())
    }
}
